# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import hashlib
import time


def hashstr(s, nr_bins):
    return str(int(hashlib.md5(s.encode('utf-8')).hexdigest(), 16) % (nr_bins - 1) + 1)


def print_run_time(func):
    def wrapper(*args, **kw):
        local_time = time.time()
        func(*args, **kw)
        print('Current function : {function}, time used : {temps}'.format(
            function=func.__name__, temps=time.time() - local_time))

    return wrapper


def get_top_k(row, k, mininum, rank=0, forbidden=[]):
    """从1*m的系数矩阵中获得top k的指标

    Args:
        row(csr_matrix) :- shape=[1, any]
        k(int) :- 选取的数目上限
        mininum(int) :- 下限
        rank(int) :- 只选择前rank列的,如果rank>0
        forbidden(iterables) :- 不可以选择的
    Returns:
        top k 数值的列号
    """
    if rank > 0:
        row = row[:, :rank]
    rec_num = min(k, row.count_nonzero())
    if rec_num < mininum:
        return []
    pair = zip(row.data, row.indices)
    indices = sorted(pair, key=lambda x: x[0], reverse=True)
    indices = [ind[1] for ind in indices if ind[1] not in forbidden]
    rec_num = min(rec_num, len(indices))
    if rec_num < mininum:
        return []
    return indices[:rec_num]


def get_top_k2(data, indices, k, mininum, rank=0, forbidden=[]):
    """从1*m的系数矩阵中获得top k的指标

    Args:
        data(np.ndarray) :- shape=(any,)
        indices(np.ndarray) :- shape=(any,)
        k(int) :- 选取的数目上限
        mininum(int) :- 下限
        rank(int) :- 只选择前rank列的,如果rank>0
        forbidden(iterables) :- 不可以选择的
    Returns:
        top k 数值的列号
    """
    rec_num = min(k, len(data))
    if rec_num < mininum:
        return []
    pair = zip(data, indices)
    indices = sorted(pair, key=lambda x: x[0], reverse=True)
    indices = [ind[1] for ind in indices if ind[1] not in forbidden and ind[1] <= rank]
    rec_num = min(rec_num, len(indices))
    if rec_num < mininum:
        return []
    return indices[:rec_num]




# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
from raven import Client


class SentryCli(object):
    def __init__(self, dsn):
        if dsn:
            self.client = Client(dsn)
        else:
            raise Exception("Sentry_dsn input error!!!")

    def check_commom_err(self, e):
        self.client.captureException(e)



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
from pyutils.yamls import read_from_yaml
import pymssql


class SqlserverCli:
    def __init__(self, host, user, password, db, port=3306):
        """
        create connection to mssql
        """
        try:
            self.db = pymssql.connect(host=host,
                                      user=user,
                                      password=password,
                                      database=db,
                                      port=port,
                                      charset="utf8")
        except Exception as e:
            raise e

    def query_mssql(self, sql):
        """
        query
        """
        with self.db.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def close_mssql(self):
        """
        close connection
        """
        self.db.close()



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import datetime
import time


def get_before_date(days=1):
    if days >= 0:
        today = datetime.date.today()
        oneday = datetime.timedelta(days=days)
        yesterday = today - oneday
        return yesterday
    else:
        return None


def get_after_date(days=1):
    if days >= 0:
        today = datetime.date.today()
        oneday = datetime.timedelta(days=days)
        yesterday = today + oneday
        return yesterday
    else:
        return None


def getYesterday():
    yesterday = datetime.date.today() + datetime.timedelta(-1)
    return yesterday


def today_time_d():
    return time.strftime("%Y-%m-%d", time.localtime())


def today_time_s():
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


def get_date_from_timstamp(timestamp):
    return datetime.datetime.fromtimestamp(timestamp)


def get_date_from_str(dtstr, format="%Y-%m-%d"):
    return datetime.datetime.strptime(dtstr, format)


def get_date_compare(dtstr1=None, dtstr2=None, format='%Y-%m-%d'):
    if dtstr1 and dtstr2:
        date1 = get_date_from_str(dtstr1, format).date()
        date2 = get_date_from_str(dtstr2, format).date()
        return date1 > date2
    return None


# 定义时间差函数
def get_date_diff(start_date: str, end_date: str):
    startTime = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
    endTime = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    diff = endTime - startTime
    return diff


def str_to_timestamp(str_time=None, format='%Y-%m-%d %H:%M:%S'):
    if str_time:
        time_tuple = time.strptime(str_time, format)  # 把格式化好的时间转换成元祖
        result = time.mktime(time_tuple)  # 把时间元祖转换成时间戳
        return int(result)
    return int(time.time())


if __name__ == "__main__":
    print(get_after_date(1))
    print(get_before_date(1))
    print(today_time_s())
    date1 = "2017-06-07"
    date2 = "2017-06-09"
    print(get_date_compare(date1, date2))



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
from rediscluster import StrictRedisCluster
import redis

class RedisClient(object):
    def __init__(self, config):
        if config["mode"] == "cluster":
            self.redis_client = StrictRedisCluster(**config["redis_cluster"])
        elif config["mode"] == "standalone":
            self.redis_client = redis.Redis(**config["redis_standalone"])
        else:
            exit("mode必须为cluster或standalone")



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import os
import json


def get_dict_from_json(filename):
    if os.path.isfile(filename):
        return json.load(open(filename, "r"), encoding="utf8")
    else:
        print(filename, " doesn't exist")
        return dict()



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import os
import re
import pandas as pd
from pyutils import dates

FILE_CSV = ".csv"
FILE_PKL = ".pkl"


class FilesOp(object):
    def __init__(self):
        pass

    def file_repalce(self, source, dest):
        os.replace(source, dest)

    def get_dir_path(self, file):
        return os.path.abspath(os.path.dirname(file))

    def get_updir_path(self, file):
        return os.path.dirname(self.get_dir_path(file))

    def get_upupdir_path(self, file):
        return os.path.dirname(self.get_updir_path(file))

    def get_dir_file_name(self, file_dir, suffix=None):
        files = list()
        for walker in os.walk(file_dir):
            if walker[2]:
                for w in walker[2]:
                    file_path = os.path.join(str(walker[0]), str(w))
                    if suffix and os.path.splitext(file_path)[1] == suffix:
                        files.append(file_path)
                    elif suffix is None:
                        files.append(file_path)
        return files

    def get_csv_data(self, file_path, *args, **kwargs):
        return pd.read_csv(file_path, *args, **kwargs)

    def get_pickle_data(self, file_path):
        return pd.read_pickle(file_path)

    def get_multi_csv_data(self, file_path, max_len=-1, *args, **kwargs):
        file_num = 0
        frames = []
        for f in self.get_dir_file_name(file_path, suffix=FILE_CSV):
            frames.append(pd.read_csv(f, *args, **kwargs))
            file_num += 1
            if max_len > 0 and file_num >= max_len:
                break
        result = pd.concat(frames)
        return result

    def get_multi_pickle_data(self, file_path, max_len=-1):
        file_num = 0
        frames = []
        for x in self.get_dir_file_name(file_path, suffix=FILE_PKL):
            frames.append(pd.read_pickle(x))
            file_num += 1
            if file_num >= max_len:
                break
        result = pd.concat(frames)
        return result

    def rm_file_by_date(self, file_path, date_len=30, suffix=FILE_CSV):
        thr_date = dates.get_before_date(date_len).strftime("%Y-%m-%d")
        for x in self.get_dir_file_name(file_path, suffix=suffix):
            file_date = re.findall(r"(\d{4}-\d{1,2}-\d{1,2})", x)
            if file_date and dates.get_date_compare(thr_date, file_date[0]):
                os.remove(x)


if __name__ == "__main__":
    # print(FilesOp().get_dir_file_name("/home/zhanglanhui/recommend_workspace/jupyter/csv/csv"))
    print(FilesOp().get_multi_csv_data("/home/zhanglanhui/recommend_workspace/jupyter/csv/csv", max_len=3,
                                       sep="$"))



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import yagmail
from functools import wraps
import traceback


class PyEmail(object):
    def __init__(self, source_mail, mail_pass, dest_mail):
        self.source_mail = source_mail
        self.mail_pass = mail_pass
        self.dest_mail = dest_mail

    def send_mail(self, sub, content):
        mail_host = "smtp.exmail.qq.com"  # 设置服务器
        mail_port = 465
        mail_user = self.source_mail  # 用户名
        mail_pass = self.mail_pass  # 口令
        server = yagmail.SMTP(mail_user, mail_pass, mail_host, mail_port)
        server.send(self.dest_mail, sub, content)

    def send_err_mail(self, sub):
        def func_wraps(func):
            @wraps(func)
            def func_process(*args, **kwargs):
                try:
                    func(*args, **kwargs)
                except Exception:
                    exstr = traceback.format_exc()
                    self.send_mail(sub, exstr)

            return func_process

        return func_wraps



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""


def get_read_file_func(spark, read_type='hive', db="ods_samh", hdfs_path="/user/hive/warehouse/ods_samh.db"):
    """
    读取数据的函数
    :param read_type: 读取方式（hive或者hdfs)
    :return:
    """
    if read_type == 'hive':
        read_func = lambda x: spark.table("{}.{}".format(db, x))
    elif read_type == 'hdfs':
        read_func = lambda x: spark.read.option("mergeSchema", "true").parquet(
            "{}/{}".format(hdfs_path, x))
    else:
        raise ValueError("wrong read file type")
    return read_func



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import json
import math
import datetime
import numpy as np
import pandas as pd
from pyutils import dates
from pyutils.list import get_flat_list
from sklearn.preprocessing import MinMaxScaler
from sklearn.preprocessing import StandardScaler


class FeaturesEncode(object):
    def __init__(self):
        pass

    def get_df_cols(self, df: pd.DataFrame):
        return set(df.columns.tolist())

    def get_nan_transform(self, df: pd.DataFrame, cols: list, type="category", value=0, method="default"):
        """
        缺省值处理，当前只支持默认值，后面需要加上其他处理方式
        :param df: DataFrame
        :param cols: 字段
        :param type: 字段数值类型
        :param method: nan处理方式
        :return:
        """
        if not set(cols).issubset(self.get_df_cols(df)):
            raise ValueError("nan transform field input not found in DataFrame!!!")

        if method == "default":
            if type == "category":
                type, value = "object", str(value)
            elif type == "numerical":
                type, value = "float32", float(value)
            elif type == "time":
                type, value = "object", "1970-01-01 00:00:00"
            else:
                raise ValueError("nan transform type input %s is not support!!!" % type)
        elif method == "linear":
            if type == "numerical":
                type, value = "float32", float(value)
                for f in cols:
                    df[f] = df[f].interpolate(method='linear', axis=0)
            else:
                raise ValueError("nan transform only support numerical when method is linear!!!")
        else:
            raise ValueError("scaler transform field input %s not found in DataFrame!!!" % str(method))

        df = df.astype({x: type for x in cols})
        df.fillna({x: value for x in cols}, inplace=True)

        return df

    def get_scaler_transform(self, df: pd.DataFrame, field, is_log=False, base=math.exp(-1), method="normalize"):
        """
        get_scaler_transform 数值型特征数据归一化
        :param df: DataFrame
        :param field: 字段
        :param is_log: 是否取log
        :param method: 归一化方式
        :return:
        """
        if field not in self.get_df_cols(df):
            raise ValueError("scaler transform field input %s not found in DataFrame!!!" % str(field))

        if is_log:
            df[field] = df[field].parallel_apply(lambda x: math.log(max(x, base)))

        if "normalize" == method:
            norm = MinMaxScaler().fit_transform(df[field].values.astype(np.float64).reshape(-1, 1))
            df[field] = norm.astype(np.float64).reshape(-1)
        elif "standard" == method:
            norm = StandardScaler().fit_transform(df[field].values.astype(np.float64).reshape(-1, 1))
            df[field] = norm.astype(np.float64).reshape(-1)
        else:
            raise ValueError("scaler transform method %s input error!!!" % str(method))
        return df

    def get_discrete_transform(self, df: pd.DataFrame, field, block=10):
        """
        get_discrete_transform 连续值离散化
        :param df: DataFrame
        :param field: 字段
        :param block: 分段
        :return: 
        """
        if field not in self.get_df_cols(df):
            raise ValueError("discrete transform field input %s not found in DataFrame!!!" % str(field))

        df[field] = df[field].multiply(block).map(int).map(str)
        return df

    def get_time2category_transform(self, df: pd.DataFrame, field, method="timestamp", dim="day"):
        """
        get_time2category_transform 时间特征处理
        :param df: DataFrame
        :param field: 字段
        :param method: 数据分析方式
        :param dim: 维度
        :return: 
        """
        if field not in self.get_df_cols(df):
            raise ValueError("time transform field input %s not found in DataFrame!!!" % str(field))

        if method == "timestamp":
            if dim == "sec":
                df[field] = df[field].parallel_apply(lambda x: dates.str_to_timestamp(x))
            elif dim == "day":
                df[field] = df[field].parallel_apply(lambda x: dates.get_date_from_str(x).day)
            elif dim == "month":
                df[field] = df[field].parallel_apply(lambda x: dates.get_date_from_str(x).month)
            else:
                raise ValueError("time transform dim input %s error!!!" % str(dim))
        elif method == "timediff":
            now_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            if dim == "sec":
                df[field] = df[field].parallel_apply(lambda x: dates.get_date_diff(start_date=str(x),
                                                                                   end_date=now_time).seconds)
            elif dim == "day":
                df[field] = df[field].parallel_apply(lambda x: dates.get_date_diff(start_date=str(x),
                                                                                   end_date=now_time).days)
            elif dim == "month":
                df[field] = df[field].parallel_apply(lambda x: int(dates.get_date_diff(start_date=str(x),
                                                                                       end_date=now_time).days / 30.0))
            else:
                raise ValueError("time transform method input %s error!!!" % str(method))
        else:
            raise ValueError("time2category transform method input error!!!")
        return df

    def get_singular_truncation_transform(self, df: pd.DataFrame, field, thr=None, method="min"):
        """
        get_singular_truncation_transform 浮点值奇异值校正
        :param df: DataFrame
        :param field: 字段
        :param thr: 阈值
        :param method: 方式
        :return:
        """
        if field not in self.get_df_cols(df):
            raise ValueError("singular transform field input %s not found in DataFrame!!!" % str(field))

        if not thr:
            return df

        if method == "min":
            df[field] = df[field].parallel_apply(lambda x: min(x, thr))
        else:
            df[field] = df[field].parallel_apply(lambda x: max(x, thr))

        return df

    def get_category_type_transform(self, df: pd.DataFrame, field, is_int: False):
        """
        get_category_type_transform
        :param df: DataFrame
        :param field: 字段
        :param is_int: 是否需要转成int
        :return:
        """
        if field not in self.get_df_cols(df):
            raise ValueError("category type transform field input %s not found in DataFrame!!!" % str(field))

        if df[field].dtype == 'float64' or df[field].dtype == 'float32' or is_int:
            df[field] = df[field].map(int).map(str)
        df[field] = df[field].astype("category")

        return df

    def get_json2category_transform(self, df: pd.DataFrame, cols: list):
        """
        json特征处理 json to category
        :param df:
        :param cols:
        :return:
        """

        def func(d):
            try:
                return "|".join(json.loads(d).keys())
            except:
                return ""

        if not cols:
            return df

        for k in cols:
            df[k] = df.parallel_apply(lambda x: func(x[k]), axis=1)

        return df

    def get_vector2category_transform(self, df: pd.DataFrame, cols: list, is_drop=True):
        """
        多维特征处理(暂时弃用)
        :param df:
        :param cols:
        :param is_drop:
        :return:
        """
        drop_features = []
        category_features_gen = []
        if not cols:
            return []
        df_vector = df
        for v in cols:
            if v in self.get_df_cols(df):
                dfTmp = df_vector[v].str.get_dummies(sep='|')
                dfTmp.rename(columns=lambda x: v + "|" + x, inplace=True)
                df_vector = df_vector.join(dfTmp)
                drop_features.append(v)
                category_features_gen.append(dfTmp.columns.tolist())
                del dfTmp
        if is_drop:
            df_vector.drop(drop_features, axis=1, inplace=True)
        return df_vector, category_features_gen

    def get_vector2value_transform(self, df: pd.DataFrame, cols: list, is_drop=True):
        """
        多维特征处理
        :param df:
        :param cols:
        :param is_drop:
        :return:
        """
        drop_features = []
        value_features_gen = []
        if not cols:
            return df, []
        df_vector = df
        for v in cols:
            if v in self.get_df_cols(df):
                val = np.array([str.split(x, "|") for x in df[v].values])
                index = [v + ":" + str(i) for i in range(1, len(val) + 1)]
                dfTmp = pd.DataFrame(val, index=index)
                df_vector = df_vector.join(dfTmp)
                drop_features.append(v)
                value_features_gen.append(dfTmp.columns.tolist())
                del dfTmp
        if is_drop:
            df_vector.drop(drop_features, axis=1, inplace=True)
        return df_vector, value_features_gen

    # 构造交叉特征
    def get_cross_transform(self, df: pd.DataFrame, field1, field2, vector_col=None):
        """
        get_cross_transform
        :param df:
        :param field1:
        :param field2:
        :param vector_col:
        :return:
        """

        def func1(df: pd.DataFrame, col1: str, col2: str):
            return '|'.join(['{}*{}'.format(x, df[col2]) for x in str.split(df[col1], "|")])

        def func2(df: pd.DataFrame, col1: str, col2: str):
            return '|'.join(['{}*{}'.format(x, y) for x in str.split(df[col1], "|")
                             for y in str.split(df[col2], "|")])

        if field1 not in self.get_df_cols(df) or field2 not in self.get_df_cols(df):
            raise ValueError("cross transform field input %s or %s not found in DataFrame!!!" % (field1, field2))

        new_field = '{}*{}'.format(field1, field2)
        if field1 in set(vector_col) and field2 in set(vector_col):
            df[new_field] = df.parallel_apply(lambda x: func2(x, field1, field2), axis=1)
        elif field1 in set(vector_col):
            df[new_field] = df.parallel_apply(lambda x: func1(x, field1, field2), axis=1)
        elif field2 in set(vector_col):
            df[new_field] = df.parallel_apply(lambda x: func1(x, field2, field1), axis=1)
        else:
            df[new_field] = df[field1].astype("object").map(str) + \
                            "*" + \
                            df[field2].astype("object").map(str)
        df[new_field] = df[new_field].astype("category")
        return df

    def gen_features_id_map(self, df: pd.DataFrame, feature_id: dict, vector, label="label"):
        """
        gen_features_id_map
        :param df:
        :param feature_id:
        :param vector:
        :param label:
        :return:
        """
        cols = self.get_df_cols(df)
        cols.remove(label)
        feature_id_list = []
        for c in cols:
            if c in vector:
                feature_id_list.extend(["{}:{}".format(c, x) for d in df[c]
                                        for x in set(get_flat_list([str.split(d, "|")]))])
            else:
                feature_id_list.extend(["{}:{}".format(c, x) for x in list(df[c].drop_duplicates())])
        max_len = max(feature_id.values()) if feature_id else 0
        for x in feature_id_list:
            if not feature_id.get(x, None):
                max_len += 1
                feature_id[x] = max_len
        return feature_id



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import pyarrow as pa
import re, os
from pyutils import dates


# 关于python操作hdfs的API可以查看官网:
# https://hdfscli.readthedocs.io/en/latest/api.html
class HDFSCli(object):
    def __init__(self, host="default", port=0, user=None):
        self.fs = pa.hdfs.connect(host, port, user=user)

    # 创建目录
    def mkdirs(self, hdfs_path):
        self.fs.mkdir(hdfs_path)

    # 删除hdfs文件
    def delete_hdfs_file(self, hdfs_path):
        self.fs.delete(hdfs_path)

    # 判断文档存在
    def is_exist_file(self, hdfs_path) -> bool:
        return self.fs.exists(hdfs_path)

    # 移动或者修改文件
    def move_or_rename(self, hdfs_src_path, hdfs_dst_path) -> bool:
        return self.fs.rename(hdfs_src_path, hdfs_dst_path)

    # 从hdfs获取文件到本地
    def get_from_hdfs(self, local_path, hdfs_path):
        self.fs.download(hdfs_path, local_path)

    # 上传文件到hdfs
    def put_to_hdfs(self, file_stream, hdfs_path):
        self.fs.upload(hdfs_path, file_stream)

    # 返回目录下的文件
    def list(self, hdfs_path):
        return self.fs.ls(hdfs_path)

    def rm_file_by_date(self, hdfs_path, date_len=30):
        thr_date = dates.get_before_date(date_len).strftime("%Y-%m-%d")
        for x in self.list(hdfs_path):
            file_date = re.findall(r"(\d{4}-\d{1,2}-\d{1,2})", x)
            if file_date and dates.get_date_compare(thr_date, file_date[0]):
                self.delete_hdfs_file(os.path.join(hdfs_path, x))

    def rm_file_all(self, hdfs_path):
        for x in self.list(hdfs_path):
            self.delete_hdfs_file(os.path.join(hdfs_path, x))


if __name__ == '__main__':
    fs = HDFSCli()
    print("root ls", fs.list("/"))



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import yaml


def read_from_yaml(file: str) -> dict:
    with open(file) as f:
        conf_dic = yaml.load(f.read(),
                             Loader=yaml.FullLoader)
    return conf_dic


def get_config_str(file: str, key: str) -> str:
    conf_dic = read_from_yaml(file)
    return conf_dic.get(key, "")


def get_config_int(file: str, key: str) -> int:
    conf_dic = read_from_yaml(file)
    return conf_dic.get(key, 0)


if __name__ == "__main__":
    print(read_from_yaml("config/dev/server_config.yaml"))



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: LiangjunFeng
# Mail: zhumavip@163.com
# Created Time:  2018-4-16 19:17:34
#############################################

from setuptools import setup, find_packages            #这个包没有的可以pip一下

setup(
    name = "rspyutils",   #这里是pip项目发布的名称
    version = "0.1.25",  #版本号，数值大的会优先被pip
    keywords = ("pip", "pyutils","featureextraction"),
    description = "An feature extraction algorithm",
    long_description = "An feature extraction algorithm, improve the FastICA",
    license = "MIT Licence",

    url = "https://github.com/zhanglanhui/pyutils",     #项目相关文件地址，一般是github
    author = "zhanglanhui",
    author_email = "zhanglanhui@xndm.tech",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = ["PyMySQL","pyhdfs","pyhive","pymssql","raven","yagmail","PyYAML"]          #这个项目需要的第三方库
)



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
hive client
"""
from pyhive import hive


class HiveCli:
    def __init__(self, host, db, port=10000, authMechanism="KERBEROS", queuename="recommend"):
        """
        create connection to hive server2
        """
        try:
            self.conn = hive.connect(host=host,
                                     port=port,
                                     auth=authMechanism,
                                     kerberos_service_name="hive",
                                     database=db,
                                     configuration={'yarn.app.mapreduce.am.resource.mb': '1024',
                                                    'mapreduce.job.queuename': queuename})
        except Exception as e:
            raise e

    def query_hive(self, sql):
        """
        query
        """
        with self.conn.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def close_hive(self):
        """
        close connection
        """
        self.conn.close()



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import math
import pymysql.cursors


class MysqlCli:
    def __init__(self, host, user, password, db, port=3306):
        """
        create connection to mysql
        """
        try:
            self.db_info = pymysql.connect(host=host,
                                           user=user,
                                           passwd=password,
                                           db=db,
                                           port=port,
                                           charset="utf8")
        except Exception as e:
            raise e

    def query_mysql(self, sql):
        """
        query
        """
        with self.db_info.cursor() as cursor:
            cursor.execute(sql)
            self.db_info.commit()
            return cursor.fetchall()

    def query_part_mysql(self, sql, max=-1, Len=10000):
        """
        query
        """
        id, fetch = 0, 1
        query_out = tuple()
        if max >= 0:
            with self.db_info.cursor() as cursor:
                while fetch and id < max:
                    cursor.execute("{} limit {},{}".format(sql, id, Len))
                    self.db_info.commit()
                    fetch = cursor.fetchall()
                    id += Len
                    if fetch:
                        query_out += fetch
        else:
            with self.db_info.cursor() as cursor:
                while fetch:
                    cursor.execute("{} limit {},{}".format(sql, id, Len))
                    self.db_info.commit()
                    fetch = cursor.fetchall()
                    id += Len
                    if fetch:
                        query_out += fetch
        return query_out

    def query_condition_mysql(self, sql, condition, key, Len=1000):
        """
        query
        """
        condition_list = list(condition)
        query_out = []
        for ind in range(0, len(condition), Len):
            get = self.query_mysql(
                f"""{sql} where {key} in 
                ({",".join(list(filter(None, map(str, condition_list[ind:ind + Len]))))})""")
            for value in get:
                query_out.append(value)
        return tuple(query_out)

    def execute_mysql(self, sql):
        """
        query
        """
        with self.db_info.cursor() as cursor:
            cursor.execute(sql)
            self.db_info.commit()

    def execute_many(self, sql, args):
        """
        executemany
        """
        with self.db_info.cursor() as cursor:
            cursor.executemany(sql, args)
            self.db_info.commit()

    def cursor_mysql(self):
        return self.db_info.cursor()

    def ping_mysql(self):
        self.db_info.ping(True)

    def commit(self):
        self.db_info.commit()

    def rollback(self):
        self.db_info.rollback()

    def update_data_by_insert(self, features: dict, key: str, field: str, table: str, times=1000):
        """
        update_data_by_insert: insert data on  DUPLICATE key 字段更新
        :param features: data
        :param key: 主键
        :param field: 更新字段
        :param table: 表名
        :param times: 次数
        :return:
        """
        if len(features) < 1:
            return

        feature_list = [[k, v] for k, v in features.items()]
        try:
            L = int(math.ceil(len(features) / times))
            for d in [feature_list[i:i + L] for i in range(0, len(feature_list), L)]:
                sql = """
                INSERT INTO 
                `{t}` (`{k}`,`{f}`) 
                VALUES {d} on DUPLICATE key 
                update `{f}`=values(`{f}`);
                """.format(t=table,
                           d=','.join(["('{}','{}')".format(x[0], x[1]) for x in d]),
                           k=key,
                           f=field)
                self.execute_mysql(sql)
        except Exception as e:
            self.db_info.rollback()
            raise e

    def update_data_by_replace(self, features: list, table: str, *keys, times=100):
        """
        整表replace更新
        :param features:
        :param key:
        :param field:
        :param table:
        :param times:
        :return:
        """
        if len(features) < 1:
            return

        try:
            L = int(math.ceil(len(features) / times))
            for d in [features[i:i + L] for i in range(0, len(features), L)]:
                sql = """
                REPLACE INTO 
                `{t}` ({k}) 
                VALUES {d};
                """.format(t=table,
                           d=','.join(["(" + ",".join(["'" + str(x) + "'" for x in dl]) + ")" for dl in d]),
                           k=','.join(keys))
                self.execute_mysql(sql)
        except Exception as e:
            self.db_info.rollback()
            raise e

    def close_mysql(self):
        """
        close connection
        """
        self.db_info.close()


if __name__ == "__main__":
    client = MysqlCli(host="172.16.1.105",
                      user="recommender",
                      password="BhRuKc!RJ64eBDQn",
                      db="samh_recommend_features")
    query_out = client.query_condition_mysql(sql="select uid,ugender from user_nrt_gender_v2",
                                             key="uid",
                                             condition=[
                                                 11331330,
                                                 11504940,
                                                 12766310,
                                                 13049640,
                                                 14659860,
                                                 15557130,
                                                 15768390,
                                                 16701840,
                                                 17199990,
                                                 17738920,
                                                 18286220,
                                                 18454980,
                                                 18601930,
                                                 19172410,
                                                 19645150,
                                                 20015760,
                                                 20018020,
                                                 20970050,
                                                 21630940,
                                                 21654260],
                                             Len=2)
    print(query_out)



# !/usr/bin/python
# -*- coding: UTF-8 -*-
"""
Created on MAY 21, 2018
@author: zlh
"""
import itertools


def get_dedup_list(l):
    """
    list自去重，保持顺序
    :param l:
    :return:
    """
    tmp = list(set(l))
    tmp.sort(key=l.index)
    return tmp


def get_first_col_list(l, n=0):
    """
    获取第n列
    :param l:
    :return:
    """
    return [str(x[n]) for x in l]


def get_diff_lists(l1, l2):
    """
    获取两个list的差集
    l1-l2
    :param l1:
    :param l2:
    :return:
    """
    return list(set(l1).difference(set(l2)))


def get_flat_list(l):
    """
    使用itertools內建模块
    [[],[],[]] -> []
    :param l:
    :return:
    """
    return list(itertools.chain.from_iterable(l))


def get_intersection_list(l1, l2):
    """
    list交集
    :param l:
    :return:
    """
    return list(set(l1).intersection(set(l2)))


def get_union_list(l1, l2):
    """
    list并集,无序
    :param l1:
    :param l2:
    :return:
    """
    return list(set(l1).union(set(l2)))


def get_merge_list(l1, l2):
    """
    list并集,有序
    :param l1:
    :param l2:
    :return:
    """
    return get_dedup_list(l1 + l2)


def check_length(l, length):
    if len(l) < length:
        raise ValueError("len of list is less then {}".format(length))
